@@ -1,5 +1,5 @@
 /* insns.h   header file for insns.c
- * $Id: insns.h,v 1.27 2002/05/14 22:38:55 hpa Exp $
+ * $Id: insns.h,v 1.30 2003/02/24 23:22:47 hpa Exp $
  *
  * The Netwide Assembler is copyright (C) 1996 Simon Tatham and
  * Julian Hall. All rights reserved. The software is
@@ -10,6 +10,15 @@
 #ifndef NASM_INSNS_H
 #define NASM_INSNS_H
 
+#include "insnsi.h"		       /* instruction opcode enum */
+
+/* max length of any instruction, register name etc. */
+#if MAX_INSLEN > 9		/* MAX_INSLEN defined in insnsi.h */
+#define MAX_KEYWORD MAX_INSLEN
+#else
+#define MAX_KEYWORD 9
+#endif
+
 struct itemplate {
     int opcode;			       /* the token, passed from "parser.c" */
     int operands;		       /* number of operands */
@@ -68,6 +77,7 @@ struct itemplate {
 #define IF_3DNOW  0x00008000UL	       /* it's a 3DNow! instruction */
 #define IF_SSE    0x00010000UL	       /* it's a SSE (KNI, MMX2) instruction */
 #define IF_SSE2   0x00020000UL	       /* it's a SSE2 instruction */
+#define IF_SSE3   0x00040000UL	       /* it's a SSE3 (PNI) instruction */
 #define IF_PMASK  0xFF000000UL	       /* the mask for processor types */
 #define IF_PLEVEL 0x0F000000UL         /* the mask for processor instr. level */
 					/* also the highest possible processor */
@@ -81,6 +91,8 @@ struct itemplate {
 #define IF_P6     0x06000000UL	       /* P6 instruction */
 #define IF_KATMAI 0x07000000UL         /* Katmai instructions */
 #define IF_WILLAMETTE 0x08000000UL         /* Willamette instructions */
+#define IF_PRESCOTT   0x09000000UL     /* Prescott instructions */
+#define IF_IA64   0x0F000000UL	       /* IA64 instructions */
 #define IF_CYRIX  0x10000000UL	       /* Cyrix-specific instruction */
 #define IF_AMD    0x20000000UL	       /* AMD-specific instruction */
 
