@@ -185,6 +185,7 @@ static int symtabsection;
 static unsigned char *stabbuf=0,*stabstrbuf=0,*stabrelbuf=0;
 static int stablen,stabstrlen,stabrellen;
 
+static struct dfmt df_stabs;
 
 void stabs_init(struct ofmt *,void *,FILE *,efunc );
 void stabs_linenum(const char *filename,long linenumber,long);
@@ -917,7 +918,7 @@ static void elf_write(void)
      * sections `.comment', `.shstrtab', `.symtab' and `.strtab',
      * then optionally relocation sections for the user sections.
      */
-    if (of_elf.current_dfmt)
+    if (of_elf.current_dfmt == &df_stabs)
       nsections=8;
     else
       nsections = 5;		       /* SHN_UNDEF and the fixed ones */
@@ -934,7 +935,7 @@ static void elf_write(void)
 	}
     }
 
-    if (of_elf.current_dfmt) {
+    if (of_elf.current_dfmt == &df_stabs) {
       /* in case the debug information is wanted, just add these three sections... */
       add_sectname("",".stab");
       add_sectname("",".stabstr");
@@ -1019,7 +1020,7 @@ static void elf_write(void)
 	elf_section_header (p - shstrtab, 9, 0, sects[i]->rel, TRUE,
 			    sects[i]->rellen, nsects+3, i+1, 4, 8);
     }
-    if (of_elf.current_dfmt) {
+    if (of_elf.current_dfmt == &df_stabs) {
       /* for debugging information, create the last three sections
 	 which are the .stab , .stabstr and .rel.stab sections respectively */
 
@@ -1268,19 +1269,24 @@ struct ofmt of_elf = {
 
 /* again, the stabs debugging stuff (code) */
 
-void stabs_init(struct ofmt *of,void *id,FILE *fp,efunc error) {
+void stabs_init(struct ofmt *of,void *id,FILE *fp,efunc error)
+{
 }
 
-void stabs_linenum(const char *filename,long linenumber,long segto) {
+void stabs_linenum(const char *filename,long linenumber,long segto)
+{
   if (!stabs_filename) {
-    stabs_filename = (char *)malloc(strlen(filename)+1);
+    stabs_filename = (char *)nasm_malloc(strlen(filename)+1);
     strcpy(stabs_filename,filename);
   } else {
     if (strcmp(stabs_filename,filename)) {
       /* yep, a memory leak...this program is one-shot anyway, so who cares...
 	 in fact, this leak comes in quite handy to maintain a list of files
 	 encountered so far in the symbol lines... */
-      stabs_filename = (char *)malloc(strlen(filename)+1);
+     
+      /* why not nasm_free(stabs_filename); we're done with the old one */
+      
+      stabs_filename = (char *)nasm_malloc(strlen(filename)+1);
       strcpy(stabs_filename,filename);
     }
   }
@@ -1288,16 +1294,20 @@ void stabs_linenum(const char *filename,long linenumber,long segto) {
   currentline=linenumber;
 }
 
-void stabs_deflabel(char *name,long segment,long offset,int is_global,char *special) {
+void stabs_deflabel(char *name,long segment,long offset,int is_global,char *special)
+{
 }
 
-void stabs_directive(const char *directive,const char *params) {
+void stabs_directive(const char *directive,const char *params)
+{
 }
 
-void stabs_typevalue(long type) {
+void stabs_typevalue(long type)
+{
 }
 
-void stabs_output(int type,void *param) {
+void stabs_output(int type,void *param)
+{
   struct symlininfo *s;
   struct linelist *el;
   if (type==TY_STABSSYMLIN) {
@@ -1305,7 +1315,7 @@ void stabs_output(int type,void *param) {
       s=(struct symlininfo *)param;
       if (strcmp( s->name,".text")) return; /* we are only interested in the text stuff */
       numlinestabs++;
-      el=(struct linelist *)malloc(sizeof(struct linelist));
+      el=(struct linelist *)nasm_malloc(sizeof(struct linelist));
       el->info.offset=s->offset;
       el->info.section=s->section;
       el->info.name=s->name;
@@ -1324,22 +1334,29 @@ void stabs_output(int type,void *param) {
   stabs_immcall=0;
 }
 
+#define WRITE_STAB(p,n_strx,n_type,n_other,n_desc,n_value) \
+  do {\
+    WRITELONG(p,n_strx); \
+    WRITECHAR(p,n_type); \
+    WRITECHAR(p,n_other); \
+    WRITESHORT(p,n_desc); \
+    WRITELONG(p,n_value); \
+  } while (0)
 
 /* for creating the .stab , .stabstr and .rel.stab sections in memory */
 
-void stabs_generate() {
+void stabs_generate(void)
+{
   int i,numfiles,strsize,numstabs=0,currfile,mainfileindex;
-  struct erel rentry;
   unsigned char *sbuf,*ssbuf,*rbuf,*sptr,*rptr;
   char **allfiles;
   int *fileidx;
 
   struct linelist *ptr;
-  struct stabentry stab;
 
   ptr=stabslines;
 
-  allfiles = (char **)malloc(numlinestabs*sizeof(char *));
+  allfiles = (char **)nasm_malloc(numlinestabs*sizeof(char *));
   for (i=0;i<numlinestabs;i++) allfiles[i]=0;
   numfiles=0;
   while (ptr) {
@@ -1358,7 +1375,7 @@ void stabs_generate() {
     ptr=ptr->next;
   }
   strsize=1;
-  fileidx = (int *)malloc(numfiles*sizeof(int));
+  fileidx = (int *)nasm_malloc(numfiles*sizeof(int));
   for (i=0;i<numfiles;i++) {
     fileidx[i]=strsize;
     strsize+=strlen(allfiles[i])+1;
@@ -1375,15 +1392,15 @@ void stabs_generate() {
 	/* worst case size of the stab buffer would be:
 		the sourcefiles changes each line, which would mean 1 SOL, 1 SYMLIN per line
 		*/
-  sbuf = (unsigned char *)malloc((numlinestabs*2+3)*sizeof(struct stabentry));
+  sbuf = (unsigned char *)nasm_malloc((numlinestabs*2+3)*sizeof(struct stabentry));
 
-  ssbuf = (unsigned char *)malloc(strsize);
+  ssbuf = (unsigned char *)nasm_malloc(strsize);
 
-  rbuf = (unsigned char *)malloc(numlinestabs*8*(2+3));
+  rbuf = (unsigned char *)nasm_malloc(numlinestabs*8*(2+3));
   rptr=rbuf;
 
   for (i=0;i<numfiles;i++) {
-    strcpy(ssbuf+fileidx[i],allfiles[i]);
+    strcpy((char *)ssbuf+fileidx[i],allfiles[i]);
   }
   ssbuf[0]=0;
 
@@ -1394,78 +1411,46 @@ void stabs_generate() {
      the source-file, the n_desc field should be set to the number
      of remaining stabs
   */
-  stab.n_strx = fileidx[0];
-  stab.n_type=0;
-  stab.n_other=0;
-  stab.n_desc=0; /* # of remaining stabs (to be determined later) */
-  stab.n_value = strlen(allfiles[0])+1;
-  memcpy(sptr,&stab,sizeof(struct stabentry));
-  sptr+=sizeof(struct stabentry);
+  WRITE_STAB(sptr, fileidx[0], 0, 0, 0, strlen(allfiles[0]+12));
 
   ptr=stabslines;
-  for (i=0;i<numfiles;i++) if (!strcmp(allfiles[i],ptr->filename)) break;
-  currfile=i;
+  numstabs = 0;
 
-  /* this is the stab for the main source file */
-  stab.n_strx = fileidx[mainfileindex];
-  stab.n_type=N_SO;
-  stab.n_other=0;
-  stab.n_desc=0;
-  stab.n_value = 0; /* strlen(allfiles[mainfileindex])+1; */
-  memcpy(sptr,&stab,sizeof(struct stabentry));
-  /* relocation stuff */
-  rentry.offset=(sptr-sbuf)+8;
-			  /* the following section+3 assumption relies on the following order of output sections:
-						0 -> dummy
-						1 -> .text
-						2 -> .data
-						3 -> .comment
-						4 -> .strtab
-			     ... this is an ugly hack and should be improved in the near future
-			  */
-   rentry.info=((ptr->info.section+3)<<8)|R_386_32;
-  memcpy(rptr,&rentry,8);
-  rptr+=8;
-  sptr+=sizeof(struct stabentry);
-
-  numstabs=1;
-  currfile=mainfileindex;
+  if ( ptr ) {
+    /* this is the stab for the main source file */
+    WRITE_STAB(sptr, fileidx[mainfileindex], N_SO, 0, 0, 0);
+    
+    /* relocation stuff */
+    /* IS THIS SANE?  WHAT DOES SECTION+3 MEAN HERE? */
+    WRITELONG(rptr, (sptr-sbuf)-4);
+    WRITELONG(rptr, ((ptr->info.section+3)<<8)|R_386_32);
+    
+    numstabs++;
+    currfile=mainfileindex;
+  }
 
   while (ptr) {
     if (strcmp(allfiles[currfile],ptr->filename)) {
       /* oops file has changed... */
       for (i=0;i<numfiles;i++) if (!strcmp(allfiles[i],ptr->filename)) break;
       currfile=i;
-      stab.n_strx = fileidx[currfile];
-      stab.n_type=N_SOL;
-      stab.n_other=0;
-      stab.n_desc=0;
-      /* stab.n_value = strlen(allfiles[currfile])+1; */
-      stab.n_value = ptr->info.offset;
-      memcpy(sptr,&stab,sizeof(struct stabentry));
-		  /* relocation stuff */
-		  rentry.offset=(sptr-sbuf)+8;
-		  rentry.info=((ptr->info.section+3)<<8)|R_386_32;
-		  memcpy(rptr,&rentry,8);
-		  rptr+=8;
-      sptr+=sizeof(struct stabentry);
+      WRITE_STAB(sptr,fileidx[currfile],N_SOL,0,0,ptr->info.offset);
       numstabs++;
+
+      /* relocation stuff */
+    /* IS THIS SANE?  WHAT DOES SECTION+3 MEAN HERE? */
+      WRITELONG(rptr, (sptr-sbuf)-4);
+      WRITELONG(rptr, ((ptr->info.section+3)<<8)|R_386_32);
     }
-    stab.n_strx=0;
-    stab.n_type=N_SLINE;
-    stab.n_other=0;
-    stab.n_desc=ptr->line;
-    stab.n_value=ptr->info.offset;
-    memcpy(sptr,&stab,sizeof(struct stabentry));
+
+    WRITE_STAB(sptr,0,N_SLINE,0,ptr->line,ptr->info.offset);
     numstabs++;
 
     /* relocation stuff */
-    rentry.offset=(sptr-sbuf)+8;
-    rentry.info=((ptr->info.section+3)<<8)|R_386_32;
-    memcpy(rptr,&rentry,8);
-    rptr+=8;
+    /* IS THIS SANE?  WHAT DOES SECTION+3 MEAN HERE? */
+    WRITELONG(rptr, (sptr-sbuf)-4);
+    WRITELONG(rptr, ((ptr->info.section+3)<<8)|R_386_32);
 
-    sptr+=sizeof(struct stabentry);
     ptr=ptr->next;
 
   }
@@ -1473,8 +1458,8 @@ void stabs_generate() {
 
   ((struct stabentry *)sbuf)->n_desc=numstabs;
 
-  free(allfiles);
-  free(fileidx);
+  nasm_free(allfiles);
+  nasm_free(fileidx);
 
   stablen = (sptr-sbuf);
   stabrellen=(rptr-rbuf);
@@ -1490,11 +1475,11 @@ void stabs_cleanup() {
   while (ptr) {
     del=ptr;
     ptr=ptr->next;
-    free(del);
+    nasm_free(del);
   }
-  if (stabbuf) free(stabbuf);
-  if (stabrelbuf) free(stabrelbuf);
-  if (stabstrbuf) free(stabstrbuf);
+  if (stabbuf) nasm_free(stabbuf);
+  if (stabrelbuf) nasm_free(stabrelbuf);
+  if (stabstrbuf) nasm_free(stabstrbuf);
 }
 
 #endif /* OF_ELF */
