@@ -15,6 +15,7 @@
 #include <ctype.h>
 
 #include "nasm.h"
+#include "insns.h"
 #include "nasmlib.h"
 #include "parser.h"
 #include "float.h"
@@ -23,18 +24,7 @@ extern int  in_abs_seg;		/* ABSOLUTE segment flag */
 extern long abs_seg;		/* ABSOLUTE segment */
 extern long abs_offset;		/* ABSOLUTE segment offset */
 
-static long reg_flags[] = {	       /* sizes and special flags */
-    0, REG8, REG_AL, REG_AX, REG8, REG8, REG16, REG16, REG8, REG_CL,
-    REG_CREG, REG_CREG, REG_CREG, REG_CR4, REG_CS, REG_CX, REG8,
-    REG16, REG8, REG_DREG, REG_DREG, REG_DREG, REG_DREG, REG_DREG,
-    REG_DREG, REG_DESS, REG_DX, REG_EAX, REG32, REG32, REG_ECX,
-    REG32, REG32, REG_DESS, REG32, REG32, REG_FSGS, REG_FSGS,
-    MMXREG, MMXREG, MMXREG, MMXREG, MMXREG, MMXREG, MMXREG, MMXREG,
-    REG16, REG16, REG_DESS, FPU0, FPUREG, FPUREG, FPUREG, FPUREG,
-    FPUREG, FPUREG, FPUREG, REG_TREG, REG_TREG, REG_TREG, REG_TREG,
-    REG_TREG,
-    XMMREG, XMMREG, XMMREG, XMMREG, XMMREG, XMMREG, XMMREG, XMMREG
-};
+#include "regflags.c"		/* List of register flags */
 
 enum {				       /* special tokens */
     S_BYTE, S_DWORD, S_FAR, S_LONG, S_NEAR, S_NOSPLIT, S_QWORD,
@@ -494,6 +484,12 @@ insn *parse_line (int pass, char *buffer, insn *result,
 	    bracket = FALSE;	       /* placate optimisers */
 	}
 
+	if((result->oprs[operand].type & FAR) && !mref &&
+	     result->opcode != I_JMP && result->opcode != I_CALL)
+	{
+	    error (ERR_NONFATAL, "invalid use of FAR operand specifier");
+	}
+
 	value = evaluate (stdscan, NULL, &tokval,
 			  &result->oprs[operand].opflags,
 			  critical, error, &hints);
@@ -579,7 +575,8 @@ insn *parse_line (int pass, char *buffer, insn *result,
 	    result->oprs[operand].hintbase = hints.base;
 	    result->oprs[operand].hinttype = hints.type;
 
-	    if (e->type <= EXPR_REG_END) {   /* this bit's a register */
+	    if (e->type && e->type <= EXPR_REG_END)   /* this bit's a register */
+	    {
 		if (e->value == 1) /* in fact it can be basereg */
 		    b = e->type;
 		else	       /* no, it has to be indexreg */
