@@ -23,8 +23,8 @@
  * content size, followed by data.
  */
 
-#include <stdlib.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <errno.h>
 #include <string.h>
 #include <unistd.h>
@@ -130,7 +130,7 @@ int main(int argc, char **argv)
     int i;
     long l;
     time_t t;
-    char tmptempl[L_tmpnam], rdbuf[10];
+    char rdbuf[10];
 
     _argv = argv;
 
@@ -203,7 +203,7 @@ int main(int argc, char **argv)
 
     case 'x':
     	if (argc < 5) {
-	    fprintf(stderr, "rdflib: required paramenter missing\n");
+	    fprintf(stderr, "rdflib: required parameter missing\n");
 	    exit(1);
 	}
     case 't':
@@ -297,7 +297,7 @@ int main(int argc, char **argv)
 	argc--;
     case 'd':		/* delete module */
     	if (argc < 4) {
-	    fprintf(stderr, "rdflib: required paramenter missing\n");
+	    fprintf(stderr, "rdflib: required parameter missing\n");
 	    exit(1);
 	}
 	
@@ -319,8 +319,7 @@ int main(int argc, char **argv)
 	    }
 	}
 	
-	tmpnam(tmptempl);
-	fptmp = fopen(tmptempl,"wb");
+	fptmp = tmpfile();
 	if (! fptmp)
 	{
 	    fprintf(stderr,"rdflib: could not open temporary file\n");
@@ -333,7 +332,7 @@ int main(int argc, char **argv)
 	l = ftell(fp);		
 	fseek(fp, 0, SEEK_SET);
 	copybytes(fp, fptmp, l);
-	freopen(tmptempl, "rb", fptmp);			/* reopen files */
+	rewind(fptmp);
 	freopen(argv[2], "wb", fp);
 	
 	while (! feof(fptmp) ) {
@@ -397,7 +396,6 @@ int main(int argc, char **argv)
 	
 	fclose(fp);
 	fclose(fptmp);
-	unlink(tmptempl);
     	break;
 
     default:
